package jdk.internal.instrumentation;

import java.util.Iterator;
import jdk.internal.org.objectweb.asm.Label;
import java.util.ArrayList;
import java.util.List;
import jdk.internal.org.objectweb.asm.tree.MethodNode;
import jdk.internal.org.objectweb.asm.MethodVisitor;

final class MethodCallInliner extends MethodVisitor
{
    private final String newClass;
    private final MethodNode inlineTarget;
    private final List<MethodCallInliner.CatchBlock> blocks;
    private boolean inlining;
    private final Logger logger;
    private final int maxLocals;
    
    public MethodCallInliner(int n, String s, MethodVisitor methodVisitor, MethodNode inlineTarget, String s2, int maxLocals, Logger logger) {
        super(327680, methodVisitor);
        this.blocks = new ArrayList();
        this.newClass = s2;
        this.inlineTarget = inlineTarget;
        this.logger = logger;
        this.maxLocals = maxLocals;
        logger.trace("MethodCallInliner: targetMethod=" + s2 + "." + inlineTarget.name + inlineTarget.desc);
    }
    
    @Override
    public void visitMethodInsn(int opcode, String owner, String s, String s2, boolean isInterface) {
        if (!this.shouldBeInlined(owner, s, s2)) {
            this.mv.visitMethodInsn(opcode, owner, s, s2, isInterface);
            return;
        }
        this.logger.trace("Inlining call to " + s + s2);
        Label label = new Label();
        this.inlining = true;
        this.inlineTarget.instructions.resetLabels();
        this.inlineTarget.accept(new MethodInliningAdapter(this, label, (opcode == 184) ? 8 : 0, s2, this.maxLocals));
        this.logger.trace("Inlining done");
        this.inlining = false;
        super.visitLabel(label);
        return;
    }
    
    private boolean shouldBeInlined(String s, String anObject, String anObject2) {
        return this.inlineTarget.desc.equals(anObject2) && this.inlineTarget.name.equals(anObject) && s.equals(this.newClass.replace('.', '/'));
    }
    
    @Override
    public void visitTryCatchBlock(Label start, Label end, Label handler, String type) {
        if (!this.inlining) {
            this.blocks.add(new MethodCallInliner.CatchBlock(start, end, handler, type));
        }
        else {
            super.visitTryCatchBlock(start, end, handler, type);
        }
    }
    
    @Override
    public void visitMaxs(int maxStack, int maxLocals) {
        for (MethodCallInliner.CatchBlock catchBlock : this.blocks) {
            super.visitTryCatchBlock(catchBlock.start, catchBlock.end, catchBlock.handler, catchBlock.type);
        }
        super.visitMaxs(maxStack, maxLocals);
    }
    
    static final class CatchBlock
    {
        final Label start;
        final Label end;
        final Label handler;
        final String type;
        
        CatchBlock(Label start, Label end, Label handler, String type) {
            this.start = start;
            this.end = end;
            this.handler = handler;
            this.type = type;
        }
    }
}
